@@ -172,6 +172,10 @@ class testclass implements TestRemoteInterface, TestRemoteInterface2, TestSignal
    {
       return "This Is A UTF-8 Name: س !!";
    }
+	public String getNameAndThrow() throws TestException
+	{
+		throw new TestException("test");
+	}
    public boolean check()
    {
       System.out.println("Being checked");
@@ -333,6 +337,22 @@ class renamedsignalhandler implements DBusSigHandler<TestSignalInterface2.TestRe
    }
 }
 
+/**
+ * Empty signal handler
+ */
+class emptysignalhandler implements DBusSigHandler<TestSignalInterface.EmptySignal>
+{
+   /** Handling a signal */
+   public void handle(TestSignalInterface.EmptySignal t)
+   {
+      if (false == test.done7) {
+         test.done7 = true;
+      } else {
+         test.fail("SignalHandler E has been run too many times");
+      }
+      System.out.println("SignalHandler E Running");
+	}
+}
 /**
  * Disconnect handler
  */
@@ -424,6 +444,14 @@ class arraysignalhandler implements DBusSigHandler<TestSignalInterface.TestArray
                !"hey".equals(t.v.get(0).a.get(3)) ||
                !"aloha".equals(t.v.get(0).a.get(4)))
             test.fail("Incorrect TestArraySignal parameters");
+			
+			if (t.m.keySet().size() != 2) test.fail("Incorrect TestArraySignal map size: should be 2, actually "+t.m.keySet().size());
+         if (!(t.m.get(new UInt32(1)).b.getValue() instanceof UInt64) ||
+               678L != ((UInt64) t.m.get(new UInt32(1)).b.getValue()).longValue() ||
+				   !(t.m.get(new UInt32(42)).b.getValue() instanceof UInt64) ||
+               789L != ((UInt64) t.m.get(new UInt32(42)).b.getValue()).longValue())
+            test.fail("Incorrect TestArraySignal parameters");
+			
       } catch (Exception e) {
          e.printStackTrace();
          test.fail("SignalHandler 2 threw an exception: "+e);
@@ -472,8 +500,21 @@ class callbackhandler implements CallbackHandler<String>
       col.setStrength(Collator.PRIMARY);
       if (0 != col.compare("This Is A UTF-8 Name: ﺱ !!", r))
          test.fail("call with callback, wrong return value");
+      if (test.done4) test.fail("Already ran callback handler");
       test.done4 = true;
    }
+   public void handleError(DBusExecutionException e)
+   {
+		System.out.println("Handling error callback: "+e+" message = '"+e.getMessage()+"'");
+		if (!(e instanceof TestException)) test.fail("Exception is of the wrong sort");
+      Collator col = Collator.getInstance();
+      col.setDecomposition(Collator.FULL_DECOMPOSITION);
+      col.setStrength(Collator.PRIMARY);
+      if (0 != col.compare("test", e.getMessage()))
+			test.fail("Exception has the wrong message");
+      if (test.done8) test.fail("Already ran callback error handler");
+		test.done8=true;
+   }
 }
 
 /**
@@ -487,6 +528,8 @@ public class test
    public static boolean done4 = false;
    public static boolean done5 = false;
    public static boolean done6 = false;
+   public static boolean done7 = false;
+   public static boolean done8 = false;
    public static void fail(String message)
    {
       System.out.println("Test Failed: "+message);
@@ -518,6 +561,7 @@ public class test
 		renamedsignalhandler rsh = new renamedsignalhandler();
       try {
          /** This registers an instance of the test class as the signal handler for the TestSignal class. */
+         clientconn.addSigHandler(TestSignalInterface.EmptySignal.class, new emptysignalhandler());
          clientconn.addSigHandler(TestSignalInterface.TestSignal.class, sigh);
          clientconn.addSigHandler(TestSignalInterface2.TestRenamedSignal.class, rsh);
          clientconn.addSigHandler(DBus.Local.Disconnected.class, new disconnecthandler(clientconn, rsh));
@@ -558,6 +602,7 @@ public class test
       System.out.println("Sending Signal");
       /** This creates an instance of the Test Signal, with the given object path, signal name and parameters, and broadcasts in on the Bus. */
       serverconn.sendSignal(new TestSignalInterface.TestSignal("/foo/bar/Wibble", "Bar", new UInt32(42)));
+      serverconn.sendSignal(new TestSignalInterface.EmptySignal("/foo/bar/Wibble"));
       serverconn.sendSignal(new TestSignalInterface2.TestRenamedSignal("/foo/bar/Wibble", "Bar", new UInt32(42)));
 
       System.out.println("These things are on the bus:");
@@ -672,6 +717,8 @@ public class test
  
       System.out.println("Doing stuff asynchronously with callback");
       clientconn.callWithCallback(tri, "getName", new callbackhandler());
+      System.out.println("Doing stuff asynchronously with callback, which throws an error");
+      clientconn.callWithCallback(tri, "getNameAndThrow", new callbackhandler());
 
       /** call something that throws */
       try {
@@ -797,7 +844,10 @@ public class test
       /** This creates an instance of the Test Signal, with the given object path, signal name and parameters, and broadcasts in on the Bus. */
       List<TestStruct2> tsl = new Vector<TestStruct2>();
       tsl.add(new TestStruct2(l, new Variant<UInt64>(new UInt64(567))));
-      serverconn.sendSignal(new TestSignalInterface.TestArraySignal("/Test", tsl));
+      Map<UInt32, TestStruct2> tsm = new HashMap<UInt32, TestStruct2>();
+      tsm.put(new UInt32(1), new TestStruct2(l, new Variant<UInt64>(new UInt64(678))));
+      tsm.put(new UInt32(42), new TestStruct2(l, new Variant<UInt64>(new UInt64(789))));
+      serverconn.sendSignal(new TestSignalInterface.TestArraySignal("/Test", tsl, tsm));
       
       System.out.println("done");
 
@@ -905,6 +955,8 @@ public class test
       if (!done4) fail("Callback handler failed to be run");
       if (!done5) fail("Signal handler R failed to be run");
       if (!done6) fail("Disconnect handler failed to be run");
+      if (!done7) fail("Signal handler E failed to be run");
+      if (!done8) fail("Error callback handler failed to be run");
       
    } catch (Exception e) {
       e.printStackTrace();
